"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// import nodejs bindings to native tensorflow,
// not required, but will speed up things drastically (python required)
require('@tensorflow/tfjs-node');

var faceapi = require('face-api.js'); // implements nodejs wrappers for HTMLCanvasElement, HTMLImageElement, ImageData


var canvas = require('canvas'); // patch nodejs environment, we need to provide an implementation of
// HTMLCanvasElement and HTMLImageElement


var Canvas = canvas.Canvas,
    Image = canvas.Image,
    ImageData = canvas.ImageData;
faceapi.env.monkeyPatch({
  Canvas: Canvas,
  Image: Image,
  ImageData: ImageData
}); // Now code!

var util = require('util');

var EventEmitter = require('events').EventEmitter;

var path = require('path');

var sharp = require('sharp');

var moment = require('moment');

var Stream = require('node-rtsp-stream');

var DEFAULT_PORT = 6789;

var FaceApiRtspStreamConsumer = function FaceApiRtspStreamConsumer() {
  var _this = this;

  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  opts = _objectSpread({
    name: opts.name || "FacialApi_RTSP_".concat(Math.round(Math.random() * 1000000))
  }, opts);
  this.options = _objectSpread({
    score: opts.score || 0.5
  }, opts);

  if (this.options && this.options.url) {
    this.stream = new Stream({
      name: this.options.name,
      streamUrl: this.options.url,
      wsPort: this.options.port || DEFAULT_PORT,
      ffmpegOptions: {
        '-stats': '',
        // Opts needs to work face detect
        '-f': 'image2',
        '-vcodec': 'mjpeg',
        '-update': 1
      }
    });

    if (this.options.jsmpegPort) {
      this.streamJsmpeg = new Stream({
        name: this.options.name,
        streamUrl: this.options.url,
        wsPort: this.options.jsmpegPort,
        ffmpegOptions: {
          '-stats': '',
          '-r': 30,
          '-f': 'mpegts',
          '-codec:v': 'mpeg1video',
          '-codec:a': 'mp2',
          '-bf': 0
        }
      });
    }
  } else {
    var message = 'Facial Api Rtsp Stream Consumer - Loading Failed!';
    this.emit('error', {
      name: this.options.name,
      message: message
    });
    throw new Error(message);
  }

  this.stream.on('exitWithError', function () {
    _this.emit('error', {
      name: _this.options.name,
      buffer: _this.buff,
      message: "RTSP Stream Exit With Error - Stream Closed - See logs!"
    });
  });
  this.buff = Buffer.from('');
};

util.inherits(FaceApiRtspStreamConsumer, EventEmitter);
FaceApiRtspStreamConsumer.prototype._load = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
  var weightsDir;
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          // Load from source
          weightsDir = this.options.weightsDir || path.join(path.resolve(), 'src', 'facial', 'weights');
          _context.next = 4;
          return faceapi.nets.tinyFaceDetector.loadFromDisk(weightsDir);

        case 4:
          _context.next = 6;
          return faceapi.nets.faceExpressionNet.loadFromDisk(weightsDir);

        case 6:
          _context.next = 8;
          return faceapi.nets.faceLandmark68Net.loadFromDisk(weightsDir);

        case 8:
          _context.next = 10;
          return faceapi.nets.ageGenderNet.loadFromDisk(weightsDir);

        case 10:
          this.emit('load', {
            name: this.options.name,
            stream: this.stream
          });
          _context.next = 16;
          break;

        case 13:
          _context.prev = 13;
          _context.t0 = _context["catch"](0);
          this.emit('error', _context.t0);

        case 16:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this, [[0, 13]]);
}));

FaceApiRtspStreamConsumer.prototype._clear = function () {
  this.buff = Buffer.from('');
  return this.buff;
};

FaceApiRtspStreamConsumer.prototype._faceDetect = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
  var _this2 = this;

  var detectedAt, img, detections, detectionsArray, promises;
  return regeneratorRuntime.wrap(function _callee3$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          detectedAt = moment().format();
          _context3.next = 3;
          return canvas.loadImage(this.buff);

        case 3:
          img = _context3.sent;
          _context3.next = 6;
          return faceapi.detectAllFaces(img, new faceapi.TinyFaceDetectorOptions()).withFaceLandmarks().withFaceExpressions().withAgeAndGender();

        case 6:
          detections = _context3.sent;
          detectionsArray = Array.isArray(detections) ? detections : [detections];

          if (!(detectionsArray.length > 0)) {
            _context3.next = 12;
            break;
          }

          promises = detectionsArray.map( /*#__PURE__*/function () {
            var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(det) {
              var score, dims, box, boxRound, out, outBuff, imgFullBuffer, imgCroppedBuffer, data, result;
              return regeneratorRuntime.wrap(function _callee2$(_context2) {
                while (1) {
                  switch (_context2.prev = _context2.next) {
                    case 0:
                      score = det.detection.score;
                      dims = det.detection.imageDims;
                      box = det.detection.box;
                      boxRound = {
                        left: Math.round(box.x < 1 ? 1 : box.x),
                        top: Math.round(box.y < 1 ? 1 : box.y),
                        width: Math.round(box.x + box.width <= dims.width ? box.width : box.x + box.width - dims.width),
                        height: Math.round(box.y + box.height < dims.height ? box.height : box.y + box.height - dims.height)
                      };

                      if (!(score < _this2.options.score && (boxRound.y < 1 || boxRound.x < 1))) {
                        _context2.next = 7;
                        break;
                      }

                      _this2._clear();

                      return _context2.abrupt("return", {
                        buffer: _this2.b
                      });

                    case 7:
                      boxRound.width = boxRound.width < 1 ? 1 : boxRound.width;
                      boxRound.height = boxRound.height < 1 ? 1 : boxRound.height;
                      out = faceapi.createCanvasFromMedia(img);
                      outBuff = out.toBuffer();
                      _context2.next = 13;
                      return sharp(outBuff).clone().toBuffer();

                    case 13:
                      imgFullBuffer = _context2.sent;
                      _context2.next = 16;
                      return sharp(outBuff).clone().extract(boxRound).toBuffer();

                    case 16:
                      imgCroppedBuffer = _context2.sent;
                      // Enviar para o backend
                      data = {
                        url: _this2.options.url || 'stream',
                        port: _this2.options.port || DEFAULT_PORT,
                        detectedAt: detectedAt,
                        image: imgFullBuffer,
                        imageCropped: imgCroppedBuffer,
                        out: out,
                        face: det
                      };
                      result = {
                        name: _this2.options.name,
                        buffer: _this2.buff,
                        data: data
                      };

                      _this2.emit('detect', result);

                      return _context2.abrupt("return", result);

                    case 21:
                    case "end":
                      return _context2.stop();
                  }
                }
              }, _callee2);
            }));

            return function (_x) {
              return _ref3.apply(this, arguments);
            };
          }());
          _context3.next = 12;
          return Promise.all(promises);

        case 12:
          this._clear();

        case 13:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee3, this);
}));
FaceApiRtspStreamConsumer.prototype.start = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
  var _this3 = this;

  return regeneratorRuntime.wrap(function _callee5$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          _context5.prev = 0;
          this.emit('start', {
            name: this.options.name,
            stream: this.stream
          });
          _context5.next = 4;
          return this._load();

        case 4:
          this.stream.on('camdata', /*#__PURE__*/function () {
            var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(data) {
              var message;
              return regeneratorRuntime.wrap(function _callee4$(_context4) {
                while (1) {
                  switch (_context4.prev = _context4.next) {
                    case 0:
                      _context4.prev = 0;

                      if (!(data.length <= 1)) {
                        _context4.next = 5;
                        break;
                      }

                      message = "Camera Data is lower than 2 bytes!";

                      _this3.emit('error', {
                        name: _this3.options.name,
                        message: message
                      });

                      throw new Error(message);

                    case 5:
                      _this3.buff = Buffer.concat([_this3.buff, data]);
                      offset = data[data.length - 2].toString(16);
                      offset2 = data[data.length - 1].toString(16);

                      if (!(offset == 'ff' && offset2 == 'd9')) {
                        _context4.next = 13;
                        break;
                      }

                      // Dados corretos para deteccao facial
                      _this3.emit('data', {
                        name: _this3.options.name,
                        buffer: _this3.buff
                      });

                      _context4.next = 12;
                      return _this3._faceDetect();

                    case 12:
                      return _context4.abrupt("return", _context4.sent);

                    case 13:
                      _this3.emit('warn', {
                        name: _this3.options.name,
                        message: "Not Match offset=".concat(offset, " != 'ff'; offset2=").concat(offset2, " != 'd9', continue consumnig...")
                      });

                      _context4.next = 16;
                      return Promise.resolve({
                        buffer: _this3.buff
                      });

                    case 16:
                      return _context4.abrupt("return", _context4.sent);

                    case 19:
                      _context4.prev = 19;
                      _context4.t0 = _context4["catch"](0);
                      console.error(_context4.t0);
                      return _context4.abrupt("return", Promise.reject(_context4.t0));

                    case 23:
                    case "end":
                      return _context4.stop();
                  }
                }
              }, _callee4, null, [[0, 19]]);
            }));

            return function (_x2) {
              return _ref5.apply(this, arguments);
            };
          }());
          _context5.next = 10;
          break;

        case 7:
          _context5.prev = 7;
          _context5.t0 = _context5["catch"](0);
          this.emit('error', _context5.t0);

        case 10:
        case "end":
          return _context5.stop();
      }
    }
  }, _callee5, this, [[0, 7]]);
}));

FaceApiRtspStreamConsumer.prototype.stop = function () {
  this.emit('stop', {
    name: this.options.name,
    buffer: this.buff
  });

  if (this.streamJsmpeg) {
    this.streamJsmpeg.stop();
  }

  this.stream.stop();

  this._clear();
};

module.exports = FaceApiRtspStreamConsumer;